# -----------------------------------------------------------------------------
# This file was autogenerated by symforce by:
#     symforce_cc_sym_stubs_codegen_test
# Do NOT modify by hand.
# -----------------------------------------------------------------------------

"""
This module wraps many of the C++ optimization classes.
"""

from __future__ import annotations

import typing

import numpy
import scipy.sparse

import lcmtypes.sym._imu_integrated_measurement_delta_t
import lcmtypes.sym._imu_integrated_measurement_t
import lcmtypes.sym._index_entry_t
import lcmtypes.sym._index_t
import lcmtypes.sym._key_t
import lcmtypes.sym._linearized_dense_factor_t
import lcmtypes.sym._optimization_iteration_t
import lcmtypes.sym._optimization_stats_t
import lcmtypes.sym._optimization_status_t
import lcmtypes.sym._optimizer_params_t
import lcmtypes.sym._sparse_matrix_structure_t
import lcmtypes.sym._values_t

import sym

__all__ = [
    "Factor",
    "ImuPreintegrator",
    "Key",
    "Linearization",
    "OptimizationStats",
    "Optimizer",
    "PreintegratedImuMeasurements",
    "Values",
    "default_optimizer_params",
    "optimize",
    "set_log_level",
]

class Factor:
    """
    A residual term for optimization.

    Created from a function and a set of Keys that act as inputs. Given a Values as an evaluation
    point, generates a linear approximation to the residual function.
    """
    @staticmethod
    @typing.overload
    def jacobian(
        jacobian_func: typing.Callable[
            [Values, list[lcmtypes.sym._index_entry_t.index_entry_t]], tuple
        ],
        keys: list[Key],
        sparse: bool = False,
    ) -> Factor:
        """
        Create from a function that computes the jacobian. The hessian will be computed using the
        Gauss Newton approximation::

            H   = J.T * J
            rhs = J.T * b

        Args:
          keys: The set of input arguments, in order, accepted by func.
          sparse: Create a sparse factor if True, dense factor if false. Defaults to dense.

        Precondition:
          The jacobian returned by jacobian_func has type scipy.sparse.csc_matrix if and only if sparse = True.
        """
    @staticmethod
    @typing.overload
    def jacobian(
        jacobian_func: typing.Callable[
            [Values, list[lcmtypes.sym._index_entry_t.index_entry_t]], tuple
        ],
        keys_to_func: list[Key],
        keys_to_optimize: list[Key],
        sparse: bool = False,
    ) -> Factor:
        """
        Create from a function that computes the jacobian. The hessian will be computed using the
        Gauss Newton approximation::

            H   = J.T * J
            rhs = J.T * b

        Args:
          keys_to_func: The set of input arguments, in order, accepted by func.
          keys_to_optimize: The set of input arguments that correspond to the derivative in func. Must be a subset of keys_to_func.
          sparse: Create a sparse factor if True, dense factor if false. Defaults to dense.

          Precondition:
            The jacobian returned by jacobian_func has type scipy.sparse.csc_matrix if and only if sparse = True.
        """
    @typing.overload
    def __init__(
        self,
        hessian_func: typing.Callable[
            [Values, list[lcmtypes.sym._index_entry_t.index_entry_t]], tuple
        ],
        keys: list[Key],
        sparse: bool = False,
    ) -> None:
        """
        Create directly from a hessian functor. This is the lowest-level constructor.

        Args:
          keys: The set of input arguments, in order, accepted by func.
          sparse: Create a sparse factor if True, dense factor if false. Defaults to dense.

        Precondition:
          The jacobian and hessian returned by hessian_func have type scipy.sparse.csc_matrix if and only if sparse = True.
        """
    @typing.overload
    def __init__(
        self,
        hessian_func: typing.Callable[
            [Values, list[lcmtypes.sym._index_entry_t.index_entry_t]], tuple
        ],
        keys_to_func: list[Key],
        keys_to_optimize: list[Key],
        sparse: bool = False,
    ) -> None:
        """
        Create directly from a hessian functor. This is the lowest-level constructor.

        Args:
          keys_to_func: The set of input arguments, in order, accepted by func.
          keys_to_optimize: The set of input arguments that correspond to the derivative in func. Must be a subset of keys_to_func.
          sparse: Create a sparse factor if True, dense factor if false. Defaults to dense.

        Precondition:
          The jacobian and hessian returned by hessian_func have type scipy.sparse.csc_matrix if and only if sparse = True.
        """
    def __repr__(self) -> str: ...
    def all_keys(self) -> list[Key]:
        """
        Get all keys required to evaluate this factor.
        """
    def is_sparse(self) -> bool:
        """
        Does this factor use a sparse jacobian/hessian matrix?
        """
    def linearize(self, arg0: Values) -> tuple:
        """
        Evaluate the factor at the given linearization point and output just the numerical values of the residual and jacobian.
        """
    def linearized_factor(
        self, values: Values
    ) -> lcmtypes.sym._linearized_dense_factor_t.linearized_dense_factor_t:
        """
        Evaluate the factor at the given linearization point and output a LinearizedDenseFactor that
        contains the numerical values of the residual, jacobian, hessian, and right-hand-side.

        This can only be called if is_sparse is false; otherwise, it will throw.
        """
    def optimized_keys(self) -> list[Key]:
        """
        Get the optimized keys for this factor.
        """

class ImuPreintegrator:
    """
    Class to on-manifold preintegrate IMU measurements for usage in a SymForce optimization problem.
    """
    def __init__(self, accel_bias: numpy.ndarray, gyro_bias: numpy.ndarray) -> None: ...
    def covariance(self) -> numpy.ndarray: ...
    def integrate_measurement(
        self,
        measured_accel: numpy.ndarray,
        measured_gyro: numpy.ndarray,
        accel_cov: numpy.ndarray,
        gyro_cov: numpy.ndarray,
        dt: float,
        epsilon: float = 2.220446049250313e-15,
    ) -> None: ...
    def preintegrated_measurements(self) -> PreintegratedImuMeasurements: ...

class Key:
    """
    Key type for Values. Contains a letter plus an integral subscript and superscript. Can construct with a letter, a letter + sub, or a letter + sub + super, but not a letter + super.
    """

    INVALID_LETTER: typing.ClassVar[str] = "\x00"
    INVALID_SUB: typing.ClassVar[int] = -9223372036854775808
    INVALID_SUPER: typing.ClassVar[int] = -9223372036854775808
    def __eq__(self, arg0: typing.Any) -> bool: ...
    def __getstate__(self) -> tuple: ...
    def __hash__(self) -> int: ...
    @typing.overload
    def __init__(self, letter: str) -> None: ...
    @typing.overload
    def __init__(self, letter: str, sub: int) -> None: ...
    @typing.overload
    def __init__(self, letter: str, sub: int, super: int) -> None: ...
    def __repr__(self) -> str: ...
    def __setstate__(self, arg0: tuple) -> None: ...
    def get_lcm_type(self) -> lcmtypes.sym._key_t.key_t: ...
    def lexical_less_than(self, arg0: Key) -> bool:
        """
        Return true if a is LESS than b, in dictionary order of the tuple (letter, sub, super).
        """
    def with_letter(self, letter: str) -> Key:
        """
        Creates a new key with a modified letter from an existing one.
        """
    def with_sub(self, sub: int) -> Key:
        """
        Creates a new key with a modified subscript from an existing one.
        """
    def with_super(self, super: int) -> Key:
        """
        Creates a new key with a modified superscript from an existing one.
        """
    @property
    def letter(self) -> str:
        """
        The letter value of the key.
        """
    @property
    def sub(self) -> int:
        """
        The subscript value of the key (INVALID_SUB if not set).
        """
    @property
    def super(self) -> int:
        """
        The superscript value of the key (INVALID_SUPER if not set).
        """

class Linearization:
    """
    Class for storing a problem linearization evaluated at a Values (i.e. a residual, jacobian, hessian, and rhs).
    """

    hessian_lower: scipy.sparse.csc_matrix
    jacobian: scipy.sparse.csc_matrix
    residual: numpy.ndarray
    rhs: numpy.ndarray
    def __getstate__(self) -> tuple: ...
    def __init__(self) -> None: ...
    def __setstate__(self, arg0: tuple) -> None: ...
    def error(self) -> float: ...
    def is_initialized(self) -> bool:
        """
        Returns whether the linearization is currently valid for the corresponding values. Accessing any of the members when this is false could result in unexpected behavior.
        """
    def linear_delta_error(
        self, x_update: numpy.ndarray, damping_vector: numpy.ndarray
    ) -> float: ...
    def reset(self) -> None:
        """
        Set to invalid.
        """
    def set_initialized(self, initialized: bool = True) -> None: ...

class OptimizationStats:
    """
    Debug stats for a full optimization run.
    """

    iterations: list[lcmtypes.sym._optimization_iteration_t.optimization_iteration_t]
    def __getstate__(self) -> tuple: ...
    def __init__(self) -> None: ...
    def __setstate__(self, arg0: tuple) -> None: ...
    def get_lcm_type(self) -> lcmtypes.sym._optimization_stats_t.optimization_stats_t: ...
    @property
    def best_index(self) -> int:
        """
        Index into iterations of the best iteration (containing the optimal Values).
        """
    @best_index.setter
    def best_index(self, arg0: int) -> None: ...
    @property
    def best_linearization(self) -> typing.Optional[Linearization]:
        """
        The linearization at best_index (at optimized_values), filled out if populate_best_linearization=True
        """
    @best_linearization.setter
    def best_linearization(self, arg1: Linearization) -> None: ...
    @property
    def cholesky_factor_sparsity(
        self,
    ) -> lcmtypes.sym._sparse_matrix_structure_t.sparse_matrix_structure_t:
        """
        Sparsity pattern of the cholesky factor L (filled out if debug_stats=True)
        """
    @cholesky_factor_sparsity.setter
    def cholesky_factor_sparsity(
        self, arg0: lcmtypes.sym._sparse_matrix_structure_t.sparse_matrix_structure_t
    ) -> None: ...
    @property
    def failure_reason(self) -> int:
        """
        If status == FAILED, why?  This should be cast to the NonlinearSolver::FailureReason enum for the nonlinear solver you used.
        """
    @failure_reason.setter
    def failure_reason(self, arg0: int) -> None: ...
    @property
    def jacobian_sparsity(
        self,
    ) -> lcmtypes.sym._sparse_matrix_structure_t.sparse_matrix_structure_t:
        """
        Sparsity pattern of the problem jacobian (filled out if debug_stats=True and include_jacobians=True)
        """
    @jacobian_sparsity.setter
    def jacobian_sparsity(
        self, arg0: lcmtypes.sym._sparse_matrix_structure_t.sparse_matrix_structure_t
    ) -> None: ...
    @property
    def linear_solver_ordering(self) -> numpy.ndarray:
        """
        Ordering used by the linear solver (filled out if debug_stats=True)
        """
    @linear_solver_ordering.setter
    def linear_solver_ordering(self, arg0: numpy.ndarray) -> None: ...
    @property
    def status(self) -> lcmtypes.sym._optimization_status_t.optimization_status_t:
        """
        What was the result of the optimization? (did it converge, fail, etc.)
        """
    @status.setter
    def status(self, arg0: lcmtypes.sym._optimization_status_t.optimization_status_t) -> None: ...

class Optimizer:
    """
    Class for optimizing a nonlinear least-squares problem specified as a list of Factors. For efficient use, create once and call Optimize() multiple times with different initial guesses, as long as the factors remain constant and the structure of the Values is identical.
    """
    def __init__(
        self,
        params: lcmtypes.sym._optimizer_params_t.optimizer_params_t,
        factors: list[Factor],
        name: str = "sym::Optimize",
        keys: list[Key] = [],
        epsilon: float = 2.220446049250313e-15,
    ) -> None: ...
    def compute_all_covariances(self, linearization: Linearization) -> dict[Key, numpy.ndarray]:
        """
        Get covariances for each optimized key at the given linearization

        May not be called before either optimize or linearize has been called.
        """
    def compute_covariances(
        self, linearization: Linearization, keys: list[Key]
    ) -> dict[Key, numpy.ndarray]:
        """
        Get covariances for the given subset of keys at the given linearization

        This version is potentially much more efficient than computing the covariances for all
        keys in the problem.

        Currently requires that `keys` corresponds to a set of keys at the start of the list of keys
        for the full problem, and in the same order.  It uses the Schur complement trick, so will be
        most efficient if the hessian is of the following form, with C block diagonal::

            A = ( B    E )
                ( E^T  C )
        """
    def compute_full_covariance(self, linearization: Linearization) -> numpy.ndarray:
        """
        Get the full problem covariance at the given linearization

        Unlike compute_covariance and compute_all_covariances, this includes the off-diagonal
        blocks, i.e. the cross-covariances between different keys.

        The ordering of entries here is the same as the ordering of the keys in the linearization,
        which can be accessed via linearization_index().

        May not be called before either optimize or linearize has been called.
        """
    def factors(self) -> list[Factor]:
        """
        Get the factors.
        """
    def keys(self) -> list[Key]:
        """
        Get the optimized keys.
        """
    def linearization_index(self) -> dict: ...
    def linearization_index_entry(self, key: Key) -> lcmtypes.sym._index_entry_t.index_entry_t: ...
    def linearize(self, values: Values) -> Linearization:
        """
        Linearize the problem around the given values.
        """
    @typing.overload
    def optimize(
        self, values: Values, num_iterations: int = -1, populate_best_linearization: bool = False
    ) -> OptimizationStats:
        """
        Optimize the given values in-place

        Args:
          num_iterations: If < 0 (the default), uses the number of iterations specified by the params at construction.

          populate_best_linearization: If true, the linearization at the best values will be filled out in the stats.

        Returns:
            The optimization stats
        """
    @typing.overload
    def optimize(
        self,
        values: Values,
        num_iterations: int,
        populate_best_linearization: bool,
        stats: OptimizationStats,
    ) -> None:
        """
        Optimize the given values in-place

        This overload takes the stats as an argument, and stores into there.  This allows users to
        avoid reallocating memory for any of the entries in the stats, for use cases where that's
        important.  If passed, stats must not be None.

        Args:
          num_iterations: If < 0 (the default), uses the number of iterations specified by the params at construction

          populate_best_linearization: If true, the linearization at the best values will be filled out in the stats

          stats: An OptimizationStats to fill out with the result - if filling out dynamically allocated fields here, will not reallocate if memory is already allocated in the required shape (e.g. for repeated calls to Optimize)
        """
    @typing.overload
    def optimize(self, values: Values, num_iterations: int, stats: OptimizationStats) -> None:
        """
        Optimize the given values in-place

        This overload takes the stats as an argument, and stores into there.  This allows users to
        avoid reallocating memory for any of the entries in the stats, for use cases where that's
        important.  If passed, stats must not be None.

        Args:
          num_iterations: If < 0 (the default), uses the number of iterations specified by the params at construction

          stats: An OptimizationStats to fill out with the result - if filling out dynamically allocated fields here, will not reallocate if memory is already allocated in the required shape (e.g. for repeated calls to Optimize)
        """
    @typing.overload
    def optimize(self, values: Values, stats: OptimizationStats) -> None:
        """
        Optimize the given values in-place

        This overload takes the stats as an argument, and stores into there.  This allows users to
        avoid reallocating memory for any of the entries in the stats, for use cases where that's
        important.  If passed, stats must not be None.

        Args:
          stats: An OptimizationStats to fill out with the result - if filling out dynamically allocated fields here, will not reallocate if memory is already allocated in the required shape (e.g. for repeated calls to Optimize)
        """
    def update_params(self, params: lcmtypes.sym._optimizer_params_t.optimizer_params_t) -> None:
        """
        Update the optimizer params.
        """

class PreintegratedImuMeasurements:
    """
    Struct of Preintegrated IMU Measurements (not including the covariance of change in orientation, velocity, and position).
    """
    class Delta:
        """
        A convenient struct that holds the Preintegrated delta.
        """

        DR: sym.Rot3
        Dp: numpy.ndarray
        Dt: float
        Dv: numpy.ndarray
        @staticmethod
        def from_lcm(
            arg0: lcmtypes.sym._imu_integrated_measurement_delta_t.imu_integrated_measurement_delta_t,
        ) -> PreintegratedImuMeasurements.Delta: ...
        def get_lcm_type(
            self,
        ) -> (
            lcmtypes.sym._imu_integrated_measurement_delta_t.imu_integrated_measurement_delta_t
        ): ...
        def roll_forward_state(
            self, pose_i: sym.Pose3, vel_i: numpy.ndarray, gravity: numpy.ndarray
        ) -> tuple[sym.Pose3, numpy.ndarray]: ...

    DR_D_gyro_bias: numpy.ndarray
    Dp_D_accel_bias: numpy.ndarray
    Dp_D_gyro_bias: numpy.ndarray
    Dv_D_accel_bias: numpy.ndarray
    Dv_D_gyro_bias: numpy.ndarray
    accel_bias: numpy.ndarray
    delta: PreintegratedImuMeasurements.Delta
    gyro_bias: numpy.ndarray
    @staticmethod
    def from_lcm(
        arg0: lcmtypes.sym._imu_integrated_measurement_t.imu_integrated_measurement_t,
    ) -> PreintegratedImuMeasurements: ...
    def __init__(self, accel_bias: numpy.ndarray, gyro_bias: numpy.ndarray) -> None: ...
    def get_bias_corrected_delta(
        self, new_accel_bias: numpy.ndarray, new_gyro_bias: numpy.ndarray
    ) -> PreintegratedImuMeasurements.Delta: ...
    def get_lcm_type(
        self,
    ) -> lcmtypes.sym._imu_integrated_measurement_t.imu_integrated_measurement_t: ...

class Values:
    """
    Efficient polymorphic data structure to store named types with a dict-like interface and
    support efficient repeated operations using a key index. Supports on-manifold optimization.

    Compatible types are given by the type_t enum. All types implement the StorageOps and
    LieGroupOps concepts, which are the core operating mechanisms in this class.
    """
    def __getstate__(self) -> bytes: ...
    @typing.overload
    def __init__(self) -> None:
        """
        Default construct as empty.
        """
    @typing.overload
    def __init__(self, msg: lcmtypes.sym._values_t.values_t) -> None:
        """
        Construct from serialized form.
        """
    def __repr__(self) -> str: ...
    def __setstate__(self, arg0: bytes) -> None: ...
    @typing.overload
    def at(self, key: Key) -> typing.Any:
        """
        Retrieve a value by key.
        """
    @typing.overload
    def at(self, entry: lcmtypes.sym._index_entry_t.index_entry_t) -> typing.Any:
        """
        Retrieve a value by index entry. This avoids a map lookup compared to at(key).
        """
    def cleanup(self) -> int:
        """
        Repack the data array to get rid of empty space from removed keys. If regularly removing
        keys, it's up to the user to call this appropriately to avoid storage growth. Returns the
        number of Scalar elements cleaned up from the data array.

        It will INVALIDATE all indices, offset increments, and pointers.
        Re-create an index with create_index().
        """
    @typing.overload
    def create_index(self, sort_by_offset: bool) -> lcmtypes.sym._index_t.index_t:
        """
        Create an index for all keys in this Values. This object can then be used
        for repeated efficient operations.

        If sort_by_offset is true, the index will be sorted by offset.  Otherwise, the ordering is
        not specified.

        An index will be INVALIDATED if the following happens:
          1) Remove() is called with a contained key, or RemoveAll() is called
          2) Cleanup() is called to re-pack the data array
        """
    @typing.overload
    def create_index(self, keys: list[Key]) -> lcmtypes.sym._index_t.index_t:
        """
        Create an index from the given ordered subset of keys. This object can then be used
        for repeated efficient operations on that subset of keys.

        If you want an index of all the keys, call `values.create_index(values.keys())`.

        An index will be INVALIDATED if the following happens:
          1) remove() is called with a contained key, or remove_all() is called
          2) cleanup() is called to re-pack the data array
        """
    def data(self) -> list[float]:
        """
        Raw data buffer.
        """
    def empty(self) -> bool:
        """
        Has zero keys.
        """
    def get_lcm_type(self, sort_keys: bool = False) -> lcmtypes.sym._values_t.values_t:
        """
        Serialize to LCM.
        """
    def has(self, key: Key) -> bool:
        """
        Return whether the key exists.
        """
    def items(self) -> dict[Key, lcmtypes.sym._index_entry_t.index_entry_t]:
        """
        Expose map type to allow iteration.
        """
    def keys(self, sort_by_offset: bool = True) -> list[Key]:
        """
        Get all keys.

        Args:
            sort_by_offset: Sorts by storage order to make iteration safer and more memory efficient
        """
    def local_coordinates(
        self, others: Values, index: lcmtypes.sym._index_t.index_t, epsilon: float
    ) -> numpy.ndarray:
        """
        Express this Values in the local coordinate of others Values, i.e., this \\ominus others

        Args:
            others: The other Values that the local coordinate is relative to
            index: Ordered list of keys to include (MUST be valid for both this and others Values)
            epsilon: Small constant to avoid singularities (do not use zero)
        """
    def num_entries(self) -> int:
        """
        Number of keys.
        """
    def remove(self, key: Key) -> bool:
        """
        Remove the given key. Only removes the index entry, does not change the data array.
        Returns true if removed, false if already not present.

        Call cleanup() to re-pack the data array.
        """
    def remove_all(self) -> None:
        """
        Remove all keys and empty out the storage.
        """
    def retract(
        self, index: lcmtypes.sym._index_t.index_t, delta: list[float], epsilon: float
    ) -> None:
        """
        Perform a retraction from an update vector.

        Args:
            index: Ordered list of keys in the delta vector
            delta: Update vector - MUST be the size of index.tangent_dim!
            epsilon: Small constant to avoid singularities (do not use zero)
        """
    @typing.overload
    def set(self, key: Key, value: float) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: float) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.Rot2) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.Rot2) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.Rot3) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.Rot3) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.Pose2) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.Pose2) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.Pose3) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.Pose3) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.Unit3) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.Unit3) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.ATANCameraCal) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.ATANCameraCal) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.DoubleSphereCameraCal) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(
        self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.DoubleSphereCameraCal
    ) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.EquirectangularCameraCal) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(
        self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.EquirectangularCameraCal
    ) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.LinearCameraCal) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(
        self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.LinearCameraCal
    ) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.PolynomialCameraCal) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(
        self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.PolynomialCameraCal
    ) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.SphericalCameraCal) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(
        self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.SphericalCameraCal
    ) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: sym.OrthographicCameraCal) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(
        self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: sym.OrthographicCameraCal
    ) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def set(self, key: Key, value: numpy.ndarray) -> bool:
        """
        Add or update a value by key. Returns true if added, false if updated.
        """
    @typing.overload
    def set(self, key: lcmtypes.sym._index_entry_t.index_entry_t, value: numpy.ndarray) -> None:
        """
        Update a value by index entry with no map lookup (compared to Set(key)). This does NOT add new values and assumes the key exists already.
        """
    @typing.overload
    def update(self, index: lcmtypes.sym._index_t.index_t, other: Values) -> None:
        """
        Efficiently update the keys given by this index from other into this. This purely copies slices of the data arrays, the index MUST be valid for both objects!
        """
    @typing.overload
    def update(
        self,
        index_this: lcmtypes.sym._index_t.index_t,
        index_other: lcmtypes.sym._index_t.index_t,
        other: Values,
    ) -> None:
        """
        Efficiently update the keys from a different structured Values, given by this index and other index. This purely copies slices of the data arrays. index_this MUST be valid for this object; index_other MUST be valid for other object.
        """
    def update_or_set(self, index: lcmtypes.sym._index_t.index_t, other: Values) -> None:
        """
        Update or add keys to this Values base on other Values of different structure.
        index MUST be valid for other.

        NOTE(alvin): it is less efficient than the Update methods below if index objects are created and cached. This method performs map lookup for each key of the index
        """

def default_optimizer_params() -> lcmtypes.sym._optimizer_params_t.optimizer_params_t:
    """
    Sensible default parameters for Optimizer.
    """

def optimize(
    params: lcmtypes.sym._optimizer_params_t.optimizer_params_t,
    factors: list[Factor],
    values: Values,
    epsilon: float = 2.220446049250313e-15,
) -> OptimizationStats:
    """
    Simple wrapper to make optimization one function call.
    """

def set_log_level(arg0: str) -> None: ...
